/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * University Copyright- Copyright (c) 1982, 1986, 1988
 * The Regents of the University of California
 * All Rights Reserved
 *
 * University Acknowledgment- Portions of this document are derived from
 * software developed by the University of California, Berkeley, and its
 * contributors.
 */

/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 *
 * Copyright 2016 RackTop Systems.
 */

/* Portions Copyright 2007 Jeremy Teo */
/* Portions Copyright 2006 Stephen P. Potter */

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <locale.h>
#include <libgen.h>
#include <stdlib.h>
#include <errno.h>
#include <netdb.h>

extern int getdomainname(char *name, int namelen);
extern int setdomainname(char *domain, int len);

#ifndef	TEXT_DOMAIN		/* should be defined by cc -D */
#define	TEXT_DOMAIN "SYS_TEST"	/* use this only if it wasn't */
#endif

static char *progname;

static void
usage(void)
{
	(void) fprintf(stderr, gettext("usage: %s [-d|-f] [hostname|domain]\n"),
	    basename(progname));
	exit(1);
}

int
main(int argc, char *argv[])
{
	int	optlet, dflag = 0, fflag = 0;
	char	*optstring = "df?";

	(void) setlocale(LC_ALL, "");
	(void) textdomain(TEXT_DOMAIN);

	progname = argv[0];

	opterr = 0;
	while ((optlet = getopt(argc, argv, optstring)) != -1) {
		switch (optlet) {
		case 'd':
			dflag++;
			break;
		case 'f':
			fflag++;
			break;
		case '?':
			usage();
		}
	}

	if (dflag && fflag)
		usage();

	/*
	 * if called with no options, just print out the hostname/domain
	 */
	if (argc <= optind) {
		char c_hostname[MAXHOSTNAMELEN];
		if (gethostname(c_hostname, sizeof (c_hostname)) < 0) {
			(void) fprintf(stderr,
			    gettext("%s: unable to obtain hostname\n"),
			    basename(progname));
			exit(1);
		}
		if (dflag || fflag) {
			char c_domain[MAXHOSTNAMELEN];
			if (getdomainname(c_domain, sizeof (c_domain)) < 0) {
				(void) fprintf(stderr,
				    gettext("%s: unable to obtain domain\n"),
				    basename(progname));
				exit(1);
			}
			/* Might not actually get a domain back */
			if (fflag && *c_domain)
				(void) fprintf(stdout, "%s.%s\n",
				    c_hostname, c_domain);
			else
				(void) fprintf(stdout, "%s\n",
				    dflag ? c_domain : c_hostname);
		} else {
			(void) fprintf(stdout, "%s\n", c_hostname);
		}
	} else {
		/*
		 * if called with an argument,
		 * we have to try to set the new hostname/domain
		 */
		if (argc > optind + 1)
			usage();	/* too many arguments */
		if (fflag)
			usage();	/* can't set fqdn this way */
		if (dflag) {
			char *domain = argv[optind];
			if (setdomainname(domain, strlen(domain)) < 0) {
				int err = errno;
				(void) fprintf(stderr,
				    gettext("%s: error setting domain: %s\n"),
				    basename(progname), strerror(err));
				exit(1);
			}
		} else {
			char *hostname = argv[optind];
			if (sethostname(hostname, strlen(hostname)) < 0) {
				int err = errno;
				(void) fprintf(stderr,
				    gettext("%s: error setting hostname: %s\n"),
				    basename(progname), strerror(err));
				exit(1);
			}
		}
	}
	return (0);
}
